import logging
from itertools import chain
from re import finditer
from sys import exit
from typing import Callable


def log_exception_and_exit(function: Callable):
    def wrapper(*args, **kwargs):
        try:
            return function(*args, **kwargs)
        except Exception as exception:
            exception_string = str(exception).strip('"')
            details = f": {exception_string}" if exception_string else ""
            message = _exception_to_sentence(exception) + details
            logging.error(message)
            exit(1)

    return wrapper


def _exception_to_sentence(exception: Exception) -> str:
    return _camel_case_to_sentence(exception.__class__.__name__)


def _title_case_to_lower_case(identifier: str) -> str:
    return identifier.lower() if identifier.istitle() else identifier


def _camel_case_to_sentence(identifier: str) -> str:
    matches = finditer(
        ".+?(?:(?<=[a-z])(?=[A-Z])|(?<=[A-Z])(?=[A-Z][a-z])|$)", identifier
    )
    capital_words = list(map(lambda match: match.group(0), matches))
    first_word = capital_words[:1]
    other_words = capital_words[1:]
    sentence = chain(first_word, map(_title_case_to_lower_case, other_words))
    return " ".join(sentence)



import logging
import logging.config
from json import dumps
from typing import Any

import click

from copernicus_marine_client.catalogue_parser.catalogue_parser import (
    CopernicusMarineCatalogue,
    filter_catalogue_with_strings,
    parse_catalogue,
)
from copernicus_marine_client.command_line_interface.utils import (
    MutuallyExclusiveOption,
)


@click.group()
def cli_group_describe() -> None:
    pass


@cli_group_describe.command(
    "describe",
    short_help="Print Copernicus Marine catalog as JSON",
    help="""
    Parse the Copernicus Marine catalog, then display a JSON-ified version of the corresponding python object.

    The default display contains information on the products, and more data
    can be displayed using the _include-<argument>_ flags (see Usage section).

    The _contains_ option allows the user to specify one or several strings to
    filter through the catalogue display. The search is performed recursively
    on all attributes of the catalogue, and the tokens only need to be
    contained in one of the attributes (i.e. not exact match).

    Example:

    \b
    > copernicus-marine describe --contains METOFFICE-GLO-SST-L4-NRT-OBS-SST-V2 --include-datasets

    \b
    > copernicus-marine describe -c METOFFICE-GLO-SST-L4-NRT-OBS-SST-V2
    """,  # noqa
)
@click.option(
    "--include-description",
    type=bool,
    is_flag=True,
    default=False,
    help="Include product description in output",
)
@click.option(
    "--include-datasets",
    type=bool,
    is_flag=True,
    default=False,
    help="Include product dataset details in output",
)
@click.option(
    "--include-keywords",
    type=bool,
    is_flag=True,
    default=False,
    help="Include product keyword details in output",
)
@click.option(
    "--contains",
    "-c",
    type=str,
    multiple=True,
    help="Filter catalogue output. Returns products with attributes"
    "matching a string token",
)
@click.option(
    "--overwrite-metadata-cache",
    cls=MutuallyExclusiveOption,
    type=bool,
    is_flag=True,
    default=False,
    help="Force to refresh the catalogue by overwriting the local cache",
    mutually_exclusive=["no_metadata_cache"],
)
@click.option(
    "--no-metadata-cache",
    cls=MutuallyExclusiveOption,
    type=bool,
    is_flag=True,
    default=False,
    help="Bypass the use of cache",
    mutually_exclusive=["overwrite_metadata_cache"],
)
@click.option(
    "--log-level",
    type=click.Choice(["DEBUG", "INFO", "WARN", "ERROR", "CRITICAL", "QUIET"]),
    default="INFO",
    help=(
        "Set the details printed to console by the command "
        "(based on standard logging library)."
    ),
)
def describe(
    include_description: bool,
    include_datasets: bool,
    include_keywords: bool,
    contains: list[str],
    overwrite_metadata_cache: bool,
    no_metadata_cache: bool,
    log_level: str,
) -> None:
    if log_level == "QUIET":
        logging.root.disabled = True
        logging.root.setLevel(level="CRITICAL")
    else:
        logging.root.setLevel(level=log_level)

    base_catalogue: CopernicusMarineCatalogue = parse_catalogue(
        overwrite_metadata_cache=overwrite_metadata_cache,
        no_metadata_cache=no_metadata_cache,
    )
    catalogue: Any
    if contains:
        filtered_catalogue = filter_catalogue_with_strings(
            base_catalogue, contains
        )
        catalogue = filtered_catalogue or {}
    else:
        catalogue = base_catalogue

    def default_filter(obj):
        attributes = obj.__dict__
        if not include_description:
            attributes.pop("description", None)
        if not include_datasets:
            attributes.pop("datasets", None)
        if not include_keywords:
            attributes.pop("keywords", None)
        return obj.__dict__

    json_dump = dumps(
        catalogue, default=default_filter, sort_keys=False, indent=2
    )
    logger = logging.getLogger("blank_logger")
    logger.warn(json_dump)


if __name__ == "__main__":
    cli_group_describe()



import base64
import configparser
import logging
import os
import pathlib
from dataclasses import dataclass
from datetime import timedelta
from netrc import netrc
from typing import Literal, Optional

import click
import lxml.html
import requests
from cachier import cachier


def load_credential_from_copernicus_marine_config_file(
    credential_type: Literal["username", "password"],
    config_filename: pathlib.Path,
) -> Optional[str]:
    config_file = open(config_filename)
    config_string = base64.standard_b64decode(config_file.read()).decode(
        "utf8"
    )
    config = configparser.RawConfigParser()
    config.read_string(config_string)
    credential = config.get("credentials", credential_type)
    if credential:
        logging.debug(f"{credential_type} loaded from {config_filename}")
    return credential


def load_credential_from_netrc_config_file(
    credential_type: Literal["username", "password"],
    config_filename: pathlib.Path,
    host: str,
) -> Optional[str]:
    authenticator = netrc(config_filename).authenticators(host=host)
    if authenticator:
        username, _, password = authenticator
        logging.debug(f"{credential_type} loaded from {config_filename}")
        return username if credential_type == "username" else password
    else:
        return None


def load_credential_from_motu_config_file(
    credential_type: Literal["username", "password"],
    config_filename: pathlib.Path,
) -> Optional[str]:
    motu_file = open(config_filename)
    motu_credential_type = "user" if credential_type == "username" else "pwd"
    config = configparser.RawConfigParser()
    config.read_string(motu_file.read())
    credential = config.get("Main", motu_credential_type)
    if credential:
        logging.debug(f"{credential_type} loaded from {config_filename}")
    return credential


def retrieve_credential_from_config_files(
    credential_type: Literal["username", "password"],
    config_file_directory: pathlib.Path,
    host: str = "default_host",
) -> Optional[str]:

    copernicus_marine_client_config_filename = pathlib.Path(
        config_file_directory, ".copernicus_marine_client_credentials"
    )
    netrc_type = "_netrc" if os.system == "win32" else ".netrc"
    netrc_filename = pathlib.Path(config_file_directory, netrc_type)
    motu_filename = pathlib.Path(
        config_file_directory, ".motuclient-python.ini"
    )
    if copernicus_marine_client_config_filename.exists():
        credential = load_credential_from_copernicus_marine_config_file(
            credential_type, copernicus_marine_client_config_filename
        )

    elif netrc_filename.exists():
        credential = load_credential_from_netrc_config_file(
            credential_type, netrc_filename, host=host
        )
    elif motu_filename.exists():
        credential = load_credential_from_motu_config_file(
            credential_type, motu_filename
        )
    else:
        credential = None
    return credential


def create_copernicus_marine_client_config_file(
    username: str,
    password: str,
    config_file_directory: pathlib.Path,
    overwrite_configuration_file: bool,
) -> None:
    config_lines = [
        "[credentials]\n",
        f"username={username}\n",
        f"password={password}\n",
    ]
    config_filename = pathlib.Path(
        config_file_directory, ".copernicus_marine_client_credentials"
    )
    if config_filename.exists() and not overwrite_configuration_file:
        click.confirm(
            f"File {config_filename} already exists, overwrite it ?",
            abort=True,
        )
    config_file = open(config_filename, "w")
    config_string = base64.b64encode(
        "".join(config_lines).encode("ascii", "strict")
    ).decode("utf8")
    config_file.write(config_string)
    config_file.close()


@dataclass
class CheckCredentialsResponse:
    error: Optional[ConnectionRefusedError]


@cachier(stale_after=timedelta(hours=5))
def check_copernicus_marine_credentials(
    username: Optional[str], password: Optional[str]
) -> CheckCredentialsResponse:
    """
    Check provided Copernicus Marine Credentials are correct.

    Parameters
    ----------
    username : str
        Copernicus Marine Username, provided for free from https://marine.copernicus.eu
    password : str
        Copernicus Marine Password, provided for free from https://marine.copernicus.eu

    """
    cmems_cas_url = "https://cmems-cas.cls.fr/cas/login"
    conn_session = requests.session()
    login_session = conn_session.get(cmems_cas_url)
    login_from_html = lxml.html.fromstring(login_session.text)
    hidden_elements_from_html = login_from_html.xpath(
        '//form//input[@type="hidden"]'
    )
    playload = {
        he.attrib["name"]: he.attrib["value"]
        for he in hidden_elements_from_html
    }
    playload["username"] = username
    playload["password"] = password
    response = conn_session.post(cmems_cas_url, data=playload)
    if response.text.find("success") == -1:
        check_credentials_response = CheckCredentialsResponse(
            error=ConnectionRefusedError(
                "Incorrect username or password.\n"
                "Learn how to recover your credentials at: "
                "https://help.marine.copernicus.eu/en/articles/"
                "4444552-i-forgot-my-username-or-my-password-what-should-i-do"
            )
        )
        logging.error(check_credentials_response.error)
        return check_credentials_response
    return CheckCredentialsResponse(error=None)


def main(
    username: str,
    password: str,
    config_file_directory: pathlib.Path,
    overwrite_configuration_file: bool,
) -> None:
    if not config_file_directory.exists():
        pathlib.Path.mkdir(config_file_directory, parents=True)
    create_copernicus_marine_client_config_file(
        username=username,
        password=password,
        config_file_directory=config_file_directory,
        overwrite_configuration_file=overwrite_configuration_file,
    )
